# test for eddy current problems

#Geometry
Object = "sphere.vol"
#The order of the elements in the mesh
Order = 5
#Set up some solver parameters
epsi = 10**-9
Tolerance = 10**-20




#Main code
#Import
import numpy as np
import cmath
import netgen.meshing as ngmeshing
from ngsolve import *


#Loading the object file
ngmesh = ngmeshing.Mesh(dim=3)
ngmesh.Load(Object)
    
#Creating the mesh and defining the element types
mesh = Mesh(Object)
mesh.Curve(5)


#Setup the finite element space
dom_nrs_metal = [0 if mat == "air" else 1 for mat in mesh.GetMaterials()]
#fes = HCurl(mesh, order=Order, dirichlet="outer", complex=True)
#fes = HCurl(mesh, order=Order, dirichlet="outer",complex=True, gradientdomains=dom_nrs_metal)
fes = HCurl(mesh, order=Order, dirichlet="outer",complex=True, flags = { "nograds" : True })

#Count the number of degrees of freedom
ndof = fes.ndof
    
#Define the vectors for the right hand side
xi = CoefficientFunction( (0,-z,y) )
cxi = CoefficientFunction( (2,0,0) )
    
#Setup the grid functions and arrays which will be used to store the solution vectors
Theta = GridFunction(fes)
Theta.Set(xi, BND)


#Test and trial functions
u = fes.TrialFunction()
v = fes.TestFunction()

#Create the bilinear form
f = LinearForm(fes)
f += SymbolicLFI(0 * InnerProduct(xi,v))
a = BilinearForm(fes,  condense=True)
#a = BilinearForm(fes)
a += SymbolicBFI( InnerProduct(curl(u),curl(v)))
a += SymbolicBFI( epsi * InnerProduct(u,v))
c = Preconditioner(a,"bddc")
a.Assemble()
f.Assemble()


# uncomment with static
f.vec.data += a.harmonic_extension_trans * f.vec
#solve
res = f.vec.CreateVector()
res.data = f.vec - a.mat * Theta.vec
inverse= CGSolver(a.mat, c.mat, precision=Tolerance, maxsteps=500)
Theta.vec.data += inverse * res
# uncomment with static
Theta.vec.data += a.inner_solve * f.vec
Theta.vec.data += a.harmonic_extension * Theta.vec



# compute the error
error = Integrate(InnerProduct(Theta-xi,Conj(Theta-xi))+InnerProduct(curl(Theta)-cxi,Conj(curl(Theta)-cxi)),mesh).real
den = Integrate(InnerProduct(xi,xi)+InnerProduct(cxi,cxi),mesh).real

print(Order, fes.ndof, error/den)
#print("degrees of freedom = ", fes.ndof)
#print("Error = ", error/den)


